==> mio-client-1.4.2.tar.gz: mio-client-1.4.2/src/mio/cli.py <==
# Copyright 2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


from mio import cfg
from mio import clean
from mio import cov
from mio import dox
from mio import results
from mio import sim
from mio import regr
from mio import install
from mio import publish
from mio import cache
from mio import new
from mio import common
from mio import init
from mio import main
from mio import help_text

import sys
import argparse
import os
import subprocess
import shutil
import yaml
from datetime import datetime
from yaml.loader import SafeLoader
import re
import random
import pathlib


uvm_levels = ["none","low","medium","high","debug"]
simulators = ["viv","mtr","vcs","xcl","qst","riv"]
commands   = ["clean", "cov", "dox", "init" ,"install", "new", "regr", "results", "sim"]


def main():
    parser       = build_parser()
    cli_args     = parser.parse_args()
    cfg.dbg      = cli_args.dbg
    common.dbg("CLI arguments: " + str(cli_args))
    cfg.cli_args = cli_args
    
    if cli_args.help:
        print_help_text()
    if cli_args.version:
        print_version()
    
    if cli_args.wd != None:
        cfg.set_pwd(cli_args.wd)
    else:
        cfg.pwd = os.getcwd()
    in_project = cache.find_project_descriptor()
    
    if not in_project:
        if (cli_args.command != "init") and (cli_args.command != "help"):
            common.fatal("Could not find 'mio.toml' project file")
        else:
            if cli_args.command == "init":
                init.new_project()
                common.exit(False)
    
    if cli_args.command == "help":
        print_cmd_help_text(cli_args)
        common.exit()
    
    cache.load_tree()
    cache.load_configuration()
    common.create_common_files()
    log_cli_args_to_disk()
    cache.scan_and_load_ip_metadata()
    
    if cli_args.command == 'init':
        init.new_ip()
        common.exit()
    if cli_args.command == 'new':
        new.menu(cfg.cli_args.template)
        common.exit()
    if cli_args.command == 'install':
        cfg.target_ip_name = cli_args.ip
        install.install_all_ips_for_target(cli_args.is_global, cli_args.username, cli_args.password)
        common.exit()
    if cli_args.command == 'publish':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        publish.publish_ip(cli_args.username, cli_args.password)
        common.exit()
    if cli_args.command == 'dox':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        dox.gen_doxygen()
        common.exit()
    if cli_args.command == 'cov':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        cov.gen_cov_report()
        common.exit()
    if cli_args.command == 'results':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        results.do_parse_results()
        common.exit()
    if cli_args.command == 'clean':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        clean.do_clean()
        common.exit()
    if cli_args.command == 'sim':
        cfg.target_ip_name = cli_args.ip
        sim_args = convert_cli_to_sim_args(cli_args)
        cache.check_target_ip()
        sim.main(sim_args)
        common.exit()
    if cli_args.command == 'regr':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        regr.main()
        common.exit()
    
    common.fatal("No command specified")


def build_parser():
    parser = argparse.ArgumentParser(prog="mio", description="", add_help=False)
    parser.add_argument("-h"   , "--help"   , help="Show this help message and exit.", action="store_true", default=False, required=False)
    parser.add_argument("-v"   , "--version", help="Print the mio version and exit." , action="store_true", default=False, required=False)
    parser.add_argument("--dbg",              help="Enable mio tracing output."      , action="store_true", default=False, required=False)
    parser.add_argument("-C"   , "--wd"     , help="Run as if mio was started in <path> instead of the current working directory.", type=pathlib.Path, required=False)
    subparsers = parser.add_subparsers(help='Command to be performed by mio', dest='command')
    
    parser_help = subparsers.add_parser('help', description="Provides documentation on specific command")
    parser_help.add_argument("cmd",  help='Command whose documentation to print', choices=commands)
    
    parser_init = subparsers.add_parser('init', description=help_text.init_help_text, add_help=False)
    
    parser_new = subparsers.add_parser('new', help=help_text.new_help_text, add_help=False)
    parser_new.add_argument('-t', '--template', help='Template name', choices=new.templates, required=False)
    
    parser_install = subparsers.add_parser('install', help=help_text.install_help_text, add_help=False)
    parser_install.add_argument('ip'              , help='Target IP'                                       )
    parser_install.add_argument("-g", "--global"  , help="Install dependencies under '~/.mio'." , action="store_true", default=False, required=False, dest="is_global")
    parser_install.add_argument('-u', "--username", help='Moore.io IP Marketplace username', required=False)
    parser_install.add_argument('-p', "--password", help='Moore.io IP Marketplace password', required=False)
    
    parser_publish = subparsers.add_parser('publish', help=help_text.publish_help_text, add_help=False)
    parser_publish.add_argument('ip'              , help='Target IP'                                       )
    parser_publish.add_argument('-u', "--username", help='Moore.io IP Marketplace username', required=False)
    parser_publish.add_argument('-p', "--password", help='Moore.io IP Marketplace password', required=False)
    
    parser_sim = subparsers.add_parser('sim', help=help_text.sim_help_text, add_help=False)
    parser_sim.add_argument('ip'               , help='Target IP'                                                                                                                               )
    parser_sim.add_argument('-t', "--test"     , help='Delete compiled IP dependencies.'                                                                                        , required=False)
    parser_sim.add_argument('-s', "--seed"     , help='Specify the seed for constrained-random testing.  If none is provided, a random one will be picked.', type=int           , required=False)
    parser_sim.add_argument('-v', "--verbosity", help='Specify the UVM verbosity level for logging: none, low, medium, high or debug.  Default: medium'    , choices=uvm_levels , required=False)
    parser_sim.add_argument('-w', "--waves"    , help='Enable wave capture to disk.'                                                                       , action="store_true", required=False)
    parser_sim.add_argument('-c', "--cov"      , help='Enable code & functional coverage capture.'                                                         , action="store_true", required=False)
    parser_sim.add_argument('-g', "--gui"      , help="Invoke the simulator's Graphical User Interface."                                                   , action="store_true", required=False)
    #parser_sim.add_argument('-p', "--prism"    , help='Enable Moore.io PRISM advanced UVM debugging.'                                                      , action="store_true", required=False)
    parser_sim.add_argument('-a', "--app"      , help='Specifies which simulator to use: viv, mtr, vcs, xcl, qst, riv.'                                    , choices=simulators , required=False)
    parser_sim.add_argument('-S'               , help='Force mio to simulate target IP.  Can be combined with -F, -C and/or -E.'                           , action="store_true", required=False)
    parser_sim.add_argument('-E'               , help='Force mio to elaborate target IP.  Can be combined with -F, -C and/or -S.'                          , action="store_true", required=False)
    parser_sim.add_argument('-C'               , help='Force mio to compile target IP.  Can be combined with -F, -E and/or -S.'                            , action="store_true", required=False)
    parser_sim.add_argument('-F'               , help='Force mio to invoke FuseSoC on core file(s).  Can be combined with -C, -E and/or -S.'               , action="store_true", required=False)
    parser_sim.add_argument('-+', "--args"     , help='Add arguments for compilation (+define+NAME[=VALUE]) or simulation (+NAME[=VALUE])).', nargs='+'    , dest='add_args'    , required=False)
    
    parser_sim = subparsers.add_parser('regr', help=help_text.regr_help_text, add_help=False)
    parser_sim.add_argument('ip'         , help='Target IP')
    parser_sim.add_argument('regr'       , help='Regression to be run.  For Test Bench IPs with multiple Test Suites, the suite must be specified. Ex: `mio regr my_ip apbxc.sanity`')
    parser_sim.add_argument('-a', "--app", help='Specifies which simulator to use: viv, mtr, vcs, xcl, qst, riv.', choices=simulators , required=False)
    parser_sim.add_argument('-d', "--dry", help='Compiles and elaborates target IP but only prints out the tests that would be run.', action="store_true", default=False , required=False)
    
    parser_clean = subparsers.add_parser('clean', help=help_text.clean_help_text, add_help=False)
    parser_clean.add_argument('ip'          , help='Target IP'                                                            )
    parser_clean.add_argument('-d', "--deep", help='Delete compiled IP external dependencies.', action="store_true", required=False)
    
    parser_results = subparsers.add_parser('results', help=help_text.results_help_text, add_help=False)
    parser_results.add_argument('ip'      , help='Target IP'      )
    parser_results.add_argument('filename', help='Report filename')
    
    parser_cov = subparsers.add_parser('cov', help=help_text.cov_help_text, add_help=False)
    parser_cov.add_argument('ip', help='Target IP')
    
    parser_dox = subparsers.add_parser('dox', help=help_text.dox_help_text, add_help=False)
    parser_dox.add_argument('ip', help='Target IP')
    
    return parser


def convert_cli_to_sim_args(cli_args):
    args = {}
    
    if not cli_args.C and not cli_args.E and not cli_args.S:
        args['cmp' ] = True
        args['elab'] = True
        args['sim' ] = True
    else:
        if cli_args.C:
            args['cmp'] = True
        else:
            args['cmp'] = False
        if cli_args.E:
            args['elab'] = True
        else:
            args['elab'] = False
        if cli_args.S:
            args['sim'] = True
        else:
            args['sim'] = False
    if cli_args.seed == None:
        args['seed'] = random.randint(1, 2147483646)
        common.dbg("Seed not specified, picked " + str(args['seed']))
    else:
        args['seed'] = cli_args.seed
    args['test_name'] = cli_args.test
    args['args'] = cli_args.add_args
    if cli_args.verbosity == None:
       args['verbosity'] = "medium"
    else:
       args['verbosity'] = cli_args.verbosity
    args['waves'] = cli_args.waves
    args['cov'] = cli_args.cov
    args['gui'] = cli_args.gui
    if cli_args.gui:
        args['waves'] = False
    
    if cli_args.app != None:
        if cli_args.app == "viv":
            args['simulator'] = "vivado"
        elif cli_args.app == "mtr":
            args['simulator'] = "mtr"
        elif cli_args.app == "vcs":
            args['simulator'] = "vcs"
        elif cli_args.app == "xcl":
            args['simulator'] = "xcelium"
        elif cli_args.app == "qst":
            args['simulator'] = "questa"
        elif cli_args.app == "riv":
            args['simulator'] = "riviera"
        else:
            common.dbg("Picked default simulator: vivado")
            args['simulator'] = "vivado"
    else:
        common.dbg("Picked default simulator: vivado")
        args['simulator'] = "vivado"
    common.dbg("Using simulator " + args['simulator'])
    
    return args


def print_help_text():
    print(help_text.main_help_text)
    common.exit(False)


def print_cmd_help_text(cli_args):
    if cli_args.cmd == "clean":
        print(help_text.clean_help_text)
    if cli_args.cmd == "cov":
        print(help_text.cov_help_text)
    if cli_args.cmd == "dox":
        print(help_text.dox_help_text)
    if cli_args.cmd == "init":
        print(help_text.init_help_text)
    if cli_args.cmd == "install":
        print(help_text.install_help_text)
    if cli_args.cmd == "new":
        print(help_text.new_help_text)
    if cli_args.cmd == "regr":
        print(help_text.regr_help_text)
    if cli_args.cmd == "results":
        print(help_text.results_help_text)
    if cli_args.cmd == "sim":
        print(help_text.sim_help_text)


def print_version():
    print(help_text.version_text)
    common.exit(False)


def log_cli_args_to_disk():
    try:
        common.create_file(cfg.commands_file_path)
        with open(cfg.commands_file_path, 'r') as yaml_file_read:
            ymlr = yaml.load(yaml_file_read, Loader=SafeLoader)
            if not ymlr:
                ymlr = {}
            ymlr[common.timestamp()] = sys.argv
            with open(cfg.commands_file_path, 'w') as yaml_file_write:
                yaml.dump(ymlr, yaml_file_write)
    except Exception as e:
        common.warning("Failed to log command history to disk: " + str(e))

==> mio-client-1.4.7.tar.gz: mio-client-1.4.7/src/mio/cli.py <==
# Copyright 2022 Datum Technology Corporation
# SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
########################################################################################################################


from mio import cfg
from mio import clean
from mio import cov
from mio import dox
from mio import results
from mio import sim
from mio import regr
from mio import install
from mio import publish
from mio import cache
from mio import new
from mio import common
from mio import init
from mio import main
from mio import help_text

import sys
import argparse
import os
import subprocess
import shutil
import yaml
from datetime import datetime
from yaml.loader import SafeLoader
import re
import random
import pathlib


uvm_levels = ["none","low","medium","high","debug"]
simulators = ["viv","mtr","vcs","xcl","qst","riv"]
commands   = ["clean", "cov", "dox", "init" ,"install", "new", "publish", "regr", "results", "sim"]


def main():
    parser       = build_parser()
    cli_args     = parser.parse_args()
    cfg.dbg      = cli_args.dbg
    common.dbg("CLI arguments: " + str(cli_args))
    cfg.cli_args = cli_args
    
    if cli_args.help:
        print_help_text()
    if cli_args.version:
        print_version()
    
    if cli_args.wd != None:
        cfg.set_pwd(cli_args.wd)
    else:
        cfg.pwd = os.getcwd()
    in_project = cache.find_project_descriptor()
    
    if not in_project:
        if (cli_args.command != "init") and (cli_args.command != "help"):
            common.fatal("Could not find 'mio.toml' project file")
        else:
            if cli_args.command == "init":
                init.new_project()
                common.exit(False)
    
    if cli_args.command == "help":
        print_cmd_help_text(cli_args)
        common.exit()
    
    cache.load_tree()
    cache.load_configuration()
    common.create_common_files()
    log_cli_args_to_disk()
    cache.scan_and_load_ip_metadata()
    
    if cli_args.command == 'init':
        init.new_ip()
        common.exit()
    if cli_args.command == 'new':
        new.menu(cfg.cli_args.template)
        common.exit()
    if cli_args.command == 'install':
        cfg.target_ip_name = cli_args.ip
        install.install_all_ips_for_target(cli_args.is_global, cli_args.username, cli_args.password)
        common.exit()
    if cli_args.command == 'publish':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        publish.publish_ip(cli_args.username, cli_args.password)
        common.exit()
    if cli_args.command == 'dox':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        dox.gen_doxygen()
        common.exit()
    if cli_args.command == 'cov':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        cov.gen_cov_report()
        common.exit()
    if cli_args.command == 'results':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        results.do_parse_results()
        common.exit()
    if cli_args.command == 'clean':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        clean.do_clean()
        common.exit()
    if cli_args.command == 'sim':
        cfg.target_ip_name = cli_args.ip
        sim_args = convert_cli_to_sim_args(cli_args)
        cache.check_target_ip()
        sim.main(sim_args)
        common.exit()
    if cli_args.command == 'regr':
        cfg.target_ip_name = cli_args.ip
        cache.check_target_ip()
        regr.main()
        common.exit()
    
    common.fatal("No command specified")


def build_parser():
    parser = argparse.ArgumentParser(prog="mio", description="", add_help=False)
    parser.add_argument("-h"   , "--help"   , help="Show this help message and exit.", action="store_true", default=False, required=False)
    parser.add_argument("-v"   , "--version", help="Print the mio version and exit." , action="store_true", default=False, required=False)
    parser.add_argument("--dbg",              help="Enable mio tracing output."      , action="store_true", default=False, required=False)
    parser.add_argument("-C"   , "--wd"     , help="Run as if mio was started in <path> instead of the current working directory.", type=pathlib.Path, required=False)
    subparsers = parser.add_subparsers(help='Command to be performed by mio', dest='command')
    
    parser_help = subparsers.add_parser('help', description="Provides documentation on specific command")
    parser_help.add_argument("cmd",  help='Command whose documentation to print', choices=commands)
    
    parser_init = subparsers.add_parser('init', description=help_text.init_help_text, add_help=False)
    
    parser_new = subparsers.add_parser('new', help=help_text.new_help_text, add_help=False)
    parser_new.add_argument('-t', '--template', help='Template name', choices=new.templates, required=False)
    
    parser_install = subparsers.add_parser('install', help=help_text.install_help_text, add_help=False)
    parser_install.add_argument('ip'              , help='Target IP'                                       )
    parser_install.add_argument("-g", "--global"  , help="Install dependencies under '~/.mio'." , action="store_true", default=False, required=False, dest="is_global")
    parser_install.add_argument('-u', "--username", help='Moore.io IP Marketplace username', required=False)
    parser_install.add_argument('-p', "--password", help='Moore.io IP Marketplace password', required=False)
    
    parser_publish = subparsers.add_parser('publish', help=help_text.publish_help_text, add_help=False)
    parser_publish.add_argument('ip'              , help='Target IP'                                       )
    parser_publish.add_argument('-u', "--username", help='Moore.io IP Marketplace username', required=False)
    parser_publish.add_argument('-p', "--password", help='Moore.io IP Marketplace password', required=False)
    
    parser_sim = subparsers.add_parser('sim', help=help_text.sim_help_text, add_help=False)
    parser_sim.add_argument('ip'               , help='Target IP'                                                                                                                               )
    parser_sim.add_argument('-t', "--test"     , help='Delete compiled IP dependencies.'                                                                                        , required=False)
    parser_sim.add_argument('-s', "--seed"     , help='Specify the seed for constrained-random testing.  If none is provided, a random one will be picked.', type=int           , required=False)
    parser_sim.add_argument('-v', "--verbosity", help='Specify the UVM verbosity level for logging: none, low, medium, high or debug.  Default: medium'    , choices=uvm_levels , required=False)
    parser_sim.add_argument('-w', "--waves"    , help='Enable wave capture to disk.'                                                                       , action="store_true", required=False)
    parser_sim.add_argument('-c', "--cov"      , help='Enable code & functional coverage capture.'                                                         , action="store_true", required=False)
    parser_sim.add_argument('-g', "--gui"      , help="Invoke the simulator's Graphical User Interface."                                                   , action="store_true", required=False)
    #parser_sim.add_argument('-p', "--prism"    , help='Enable Moore.io PRISM advanced UVM debugging.'                                                      , action="store_true", required=False)
    parser_sim.add_argument('-a', "--app"      , help='Specifies which simulator to use: viv, mtr, vcs, xcl, qst, riv.'                                    , choices=simulators , required=False)
    parser_sim.add_argument('-S'               , help='Force mio to simulate target IP.  Can be combined with -F, -C and/or -E.'                           , action="store_true", required=False)
    parser_sim.add_argument('-E'               , help='Force mio to elaborate target IP.  Can be combined with -F, -C and/or -S.'                          , action="store_true", required=False)
    parser_sim.add_argument('-C'               , help='Force mio to compile target IP.  Can be combined with -F, -E and/or -S.'                            , action="store_true", required=False)
    parser_sim.add_argument('-F'               , help='Force mio to invoke FuseSoC on core file(s).  Can be combined with -C, -E and/or -S.'               , action="store_true", required=False)
    parser_sim.add_argument('-+', "--args"     , help='Add arguments for compilation (+define+NAME[=VALUE]) or simulation (+NAME[=VALUE])).', nargs='+'    , dest='add_args'    , required=False)
    
    parser_sim = subparsers.add_parser('regr', help=help_text.regr_help_text, add_help=False)
    parser_sim.add_argument('ip'         , help='Target IP')
    parser_sim.add_argument('regr'       , help='Regression to be run.  For Test Bench IPs with multiple Test Suites, the suite must be specified. Ex: `mio regr my_ip apbxc.sanity`')
    parser_sim.add_argument('-a', "--app", help='Specifies which simulator to use: viv, mtr, vcs, xcl, qst, riv.', choices=simulators , required=False)
    parser_sim.add_argument('-d', "--dry", help='Compiles and elaborates target IP but only prints out the tests that would be run.', action="store_true", default=False , required=False)
    
    parser_clean = subparsers.add_parser('clean', help=help_text.clean_help_text, add_help=False)
    parser_clean.add_argument('ip'          , help='Target IP'                                                            )
    parser_clean.add_argument('-d', "--deep", help='Delete compiled IP external dependencies.', action="store_true", required=False)
    
    parser_results = subparsers.add_parser('results', help=help_text.results_help_text, add_help=False)
    parser_results.add_argument('ip'      , help='Target IP'      )
    parser_results.add_argument('filename', help='Report filename')
    
    parser_cov = subparsers.add_parser('cov', help=help_text.cov_help_text, add_help=False)
    parser_cov.add_argument('ip', help='Target IP')
    
    parser_dox = subparsers.add_parser('dox', help=help_text.dox_help_text, add_help=False)
    parser_dox.add_argument('ip', help='Target IP')
    
    return parser


def convert_cli_to_sim_args(cli_args):
    args = {}
    
    if not cli_args.C and not cli_args.E and not cli_args.S:
        args['cmp' ] = True
        args['elab'] = True
        args['sim' ] = True
    else:
        if cli_args.C:
            args['cmp'] = True
        else:
            args['cmp'] = False
        if cli_args.E:
            args['elab'] = True
        else:
            args['elab'] = False
        if cli_args.S:
            args['sim'] = True
        else:
            args['sim'] = False
    if cli_args.seed == None:
        args['seed'] = random.randint(1, 2147483646)
        common.dbg("Seed not specified, picked " + str(args['seed']))
    else:
        args['seed'] = cli_args.seed
    args['test_name'] = cli_args.test
    args['args'] = cli_args.add_args
    if cli_args.verbosity == None:
       args['verbosity'] = "medium"
    else:
       args['verbosity'] = cli_args.verbosity
    args['waves'] = cli_args.waves
    args['cov'] = cli_args.cov
    args['gui'] = cli_args.gui
    if cli_args.gui:
        args['waves'] = False
    
    if cli_args.app != None:
        if cli_args.app == "viv":
            args['simulator'] = "vivado"
        elif cli_args.app == "mtr":
            args['simulator'] = "mtr"
        elif cli_args.app == "vcs":
            args['simulator'] = "vcs"
        elif cli_args.app == "xcl":
            args['simulator'] = "xcelium"
        elif cli_args.app == "qst":
            args['simulator'] = "questa"
        elif cli_args.app == "riv":
            args['simulator'] = "riviera"
        else:
            common.dbg("Picked default simulator: vivado")
            args['simulator'] = "vivado"
    else:
        common.dbg("Picked default simulator: vivado")
        args['simulator'] = "vivado"
    common.dbg("Using simulator " + args['simulator'])
    
    return args


def print_help_text():
    print(help_text.main_help_text)
    common.exit(False)


def print_cmd_help_text(cli_args):
    if cli_args.cmd == "clean":
        print(help_text.clean_help_text)
    if cli_args.cmd == "cov":
        print(help_text.cov_help_text)
    if cli_args.cmd == "dox":
        print(help_text.dox_help_text)
    if cli_args.cmd == "init":
        print(help_text.init_help_text)
    if cli_args.cmd == "install":
        print(help_text.install_help_text)
    if cli_args.cmd == "new":
        print(help_text.new_help_text)
    if cli_args.cmd == "publish":
        print(help_text.publish_help_text)
    if cli_args.cmd == "regr":
        print(help_text.regr_help_text)
    if cli_args.cmd == "results":
        print(help_text.results_help_text)
    if cli_args.cmd == "sim":
        print(help_text.sim_help_text)


def print_version():
    print(help_text.version_text)
    common.exit(False)


def log_cli_args_to_disk():
    try:
        common.create_file(cfg.commands_file_path)
        with open(cfg.commands_file_path, 'r') as yaml_file_read:
            ymlr = yaml.load(yaml_file_read, Loader=SafeLoader)
            if not ymlr:
                ymlr = {}
            ymlr[common.timestamp()] = sys.argv
            with open(cfg.commands_file_path, 'w') as yaml_file_write:
                yaml.dump(ymlr, yaml_file_write)
    except Exception as e:
        common.warning("Failed to log command history to disk: " + str(e))
